@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 """
 base execnet gateway code send to the other side for bootstrapping.
 
@@ -12,11 +13,12 @@ NOTE: aims to be compatible to Python 2.5-3.X, Jython and IronPython
     - many others
 """
 from __future__ import with_statement
-import sys
+
 import os
-import weakref
-import traceback
 import struct
+import sys
+import traceback
+import weakref
 
 # NOTE that we want to avoid try/except style importing
 # to avoid setting sys.exc_info() during import
@@ -25,16 +27,24 @@ import struct
 ISPY3 = sys.version_info >= (3, 0)
 if ISPY3:
     from io import BytesIO
-    exec("def do_exec(co, loc): exec(co, loc)\n"
-         "def reraise(cls, val, tb): raise val\n")
+
+    exec("do_exec = exec")
+
+    def reraise(cls, val, tb):
+        raise val.with_traceback(tb)
+
     unicode = str
     _long_type = int
     from _thread import interrupt_main
+
     SUBPROCESS32 = False
 else:
     from StringIO import StringIO as BytesIO
-    exec("def do_exec(co, loc): exec co in loc\n"
-         "def reraise(cls, val, tb): raise cls, val, tb\n")
+
+    exec(
+        "def do_exec(co, loc): exec co in loc\n"
+        "def reraise(cls, val, tb): raise cls, val, tb\n"
+    )
     bytes = str
     _long_type = long
     try:
@@ -43,6 +53,7 @@ else:
         interrupt_main = None
     try:
         import subprocess32  # NOQA
+
         SUBPROCESS32 = True
     except ImportError:
         SUBPROCESS32 = False
@@ -63,17 +74,19 @@ def get_execmodel(backend):
         return backend
     if backend == "thread":
         importdef = {
-            'get_ident': ['thread::get_ident', '_thread::get_ident'],
-            '_start_new_thread': ['thread::start_new_thread',
-                                  '_thread::start_new_thread'],
-            'threading': ["threading"],
-            'queue': ["queue" if ISPY3 else "Queue"],
-            'sleep': ['time::sleep'],
-            'subprocess': ['subprocess32' if SUBPROCESS32 else 'subprocess'],
-            'socket': ['socket'],
-            '_fdopen': ['os::fdopen'],
-            '_lock': ['threading'],
-            '_event': ['threading'],
+            "get_ident": ["thread::get_ident", "_thread::get_ident"],
+            "_start_new_thread": [
+                "thread::start_new_thread",
+                "_thread::start_new_thread",
+            ],
+            "threading": ["threading"],
+            "queue": ["queue" if ISPY3 else "Queue"],
+            "sleep": ["time::sleep"],
+            "subprocess": ["subprocess32" if SUBPROCESS32 else "subprocess"],
+            "socket": ["socket"],
+            "_fdopen": ["os::fdopen"],
+            "_lock": ["threading"],
+            "_event": ["threading"],
         }
 
         def exec_start(self, func, args=()):
@@ -81,39 +94,41 @@ def get_execmodel(backend):
 
     elif backend == "eventlet":
         importdef = {
-            'get_ident': ['eventlet.green.thread::get_ident'],
-            '_spawn_n': ['eventlet::spawn_n'],
-            'threading': ['eventlet.green.threading'],
-            'queue': ["eventlet.queue"],
-            'sleep': ['eventlet::sleep'],
-            'subprocess': ['eventlet.green.subprocess'],
-            'socket': ['eventlet.green.socket'],
-            '_fdopen': ['eventlet.green.os::fdopen'],
-            '_lock': ['eventlet.green.threading'],
-            '_event': ['eventlet.green.threading'],
+            "get_ident": ["eventlet.green.thread::get_ident"],
+            "_spawn_n": ["eventlet::spawn_n"],
+            "threading": ["eventlet.green.threading"],
+            "queue": ["eventlet.queue"],
+            "sleep": ["eventlet::sleep"],
+            "subprocess": ["eventlet.green.subprocess"],
+            "socket": ["eventlet.green.socket"],
+            "_fdopen": ["eventlet.green.os::fdopen"],
+            "_lock": ["eventlet.green.threading"],
+            "_event": ["eventlet.green.threading"],
         }
 
         def exec_start(self, func, args=()):
             self._spawn_n(func, *args)
+
     elif backend == "gevent":
         importdef = {
-            'get_ident': ['gevent.thread::get_ident'],
-            '_spawn_n': ['gevent::spawn'],
-            'threading': ['threading'],
-            'queue': ["gevent.queue"],
-            'sleep': ['gevent::sleep'],
-            'subprocess': ['gevent.subprocess'],
-            'socket': ['gevent.socket'],
+            "get_ident": ["gevent.thread::get_ident"],
+            "_spawn_n": ["gevent::spawn"],
+            "threading": ["threading"],
+            "queue": ["gevent.queue"],
+            "sleep": ["gevent::sleep"],
+            "subprocess": ["gevent.subprocess"],
+            "socket": ["gevent.socket"],
             # XXX
-            '_fdopen': ['gevent.fileobject::FileObjectThread'],
-            '_lock': ['gevent.lock'],
-            '_event': ['gevent.event'],
+            "_fdopen": ["gevent.fileobject::FileObjectThread"],
+            "_lock": ["gevent.lock"],
+            "_event": ["gevent.event"],
         }
 
         def exec_start(self, func, args=()):
             self._spawn_n(func, *args)
+
     else:
-        raise ValueError("unknown execmodel %r" % (backend,))
+        raise ValueError("unknown execmodel {!r}".format(backend))
 
     class ExecModel:
         def __init__(self, name):
@@ -162,16 +177,7 @@ def get_execmodel(backend):
             return self._lock.RLock()
 
         def Event(self):
-            event = self._event.Event()
-            if sys.version_info < (2, 7):
-                # patch wait function to return event state instead of None
-                real_wait = event.wait
-
-                def wait(timeout=None):
-                    real_wait(timeout=timeout)
-                    return event.isSet()
-                event.wait = wait
-            return event
+            return self._event.Event()
 
         def PopenPiped(self, args):
             PIPE = self.subprocess.PIPE
@@ -185,6 +191,7 @@ class Reply(object):
         of a function execution that got dispatched
         through WorkerPool.spawn()
     """
+
     def __init__(self, task, threadmodel):
         self.task = task
         self._result_ready = threadmodel.Event()
@@ -204,7 +211,7 @@ class Reply(object):
 
     def waitfinish(self, timeout=None):
         if not self._result_ready.wait(timeout):
-            raise IOError("timeout waiting for %r" % (self.task, ))
+            raise IOError("timeout waiting for {!r}".format(self.task))
 
     def run(self):
         func, args, kwargs = self.task
@@ -230,6 +237,7 @@ class WorkerPool(object):
         calling integrate_as_primary_thread() which will return
         when the pool received a trigger_shutdown().
     """
+
     def __init__(self, execmodel, hasprimary=False):
         """ by default allow unlimited number of spawns. """
         self.execmodel = execmodel
@@ -328,22 +336,26 @@ class WorkerPool(object):
 sysex = (KeyboardInterrupt, SystemExit)
 
 
-DEBUG = os.environ.get('EXECNET_DEBUG')
+DEBUG = os.environ.get("EXECNET_DEBUG")
 pid = os.getpid()
-if DEBUG == '2':
+if DEBUG == "2":
+
     def trace(*msg):
         try:
             line = " ".join(map(str, msg))
-            sys.stderr.write("[%s] %s\n" % (pid, line))
+            sys.stderr.write("[{}] {}\n".format(pid, line))
             sys.stderr.flush()
         except Exception:
             pass  # nothing we can do, likely interpreter-shutdown
+
+
 elif DEBUG:
     import tempfile
     import os
-    fn = os.path.join(tempfile.gettempdir(), 'execnet-debug-%d' % pid)
+
+    fn = os.path.join(tempfile.gettempdir(), "execnet-debug-%d" % pid)
     # sys.stderr.write("execnet-debug at %r" % (fn,))
-    debugfile = open(fn, 'w')
+    debugfile = open(fn, "w")
 
     def trace(*msg):
         try:
@@ -353,10 +365,11 @@ elif DEBUG:
         except Exception:
             try:
                 v = sys.exc_info()[1]
-                sys.stderr.write(
-                    "[%s] exception during tracing: %r\n" % (pid, v))
+                sys.stderr.write("[{}] exception during tracing: {!r}\n".format(pid, v))
             except Exception:
                 pass  # nothing we can do, likely interpreter-shutdown
+
+
 else:
     notrace = trace = lambda *msg: None
 
@@ -369,6 +382,7 @@ class Popen2IO:
         self.outfile, self.infile = outfile, infile
         if sys.platform == "win32":
             import msvcrt
+
             try:
                 msvcrt.setmode(infile.fileno(), os.O_BINARY)
                 msvcrt.setmode(outfile.fileno(), os.O_BINARY)
@@ -383,10 +397,9 @@ class Popen2IO:
         # a file in non-blocking mode may return less bytes, so we loop
         buf = bytes()
         while numbytes > len(buf):
-            data = self._read(numbytes-len(buf))
+            data = self._read(numbytes - len(buf))
             if not data:
-                raise EOFError(
-                    "expected %d bytes, got %d" % (numbytes, len(buf)))
+                raise EOFError("expected %d bytes, got %d" % (numbytes, len(buf)))
             buf += data
         return buf
 
@@ -405,9 +418,10 @@ class Popen2IO:
 
 class Message:
     """ encapsulates Messages and their wire protocol. """
+
     _types = []
 
-    def __init__(self, msgcode, channelid=0, data=''):
+    def __init__(self, msgcode, channelid=0, data=""):
         self.msgcode = msgcode
         self.channelid = channelid
         self.data = data
@@ -420,22 +434,22 @@ class Message:
                 raise EOFError("empty read")
         except EOFError:
             e = sys.exc_info()[1]
-            raise EOFError('couldnt load message header, ' + e.args[0])
-        msgtype, channel, payload = struct.unpack('!bii', header)
+            raise EOFError("couldnt load message header, " + e.args[0])
+        msgtype, channel, payload = struct.unpack("!bii", header)
         return Message(msgtype, channel, io.read(payload))
 
     def to_io(self, io):
-        header = struct.pack('!bii', self.msgcode, self.channelid,
-                             len(self.data))
-        io.write(header+self.data)
+        header = struct.pack("!bii", self.msgcode, self.channelid, len(self.data))
+        io.write(header + self.data)
 
     def received(self, gateway):
         self._types[self.msgcode](self, gateway)
 
     def __repr__(self):
         name = self._types[self.msgcode].__name__.upper()
-        return "<Message %s channel=%s lendata=%s>" % (
-            name, self.channelid, len(self.data))
+        return "<Message {} channel={} lendata={}>".format(
+            name, self.channelid, len(self.data)
+        )
 
 
 class GatewayReceivedTerminate(Exception):
@@ -447,12 +461,11 @@ def _setupmessages():
         # we use the channelid to send back information
         # but don't instantiate a channel object
         d = {
-            'numchannels': len(gateway._channelfactory._channels),
-            'numexecuting': gateway._execpool.active_count(),
-            'execmodel': gateway.execmodel.backend,
+            "numchannels": len(gateway._channelfactory._channels),
+            "numexecuting": gateway._execpool.active_count(),
+            "execmodel": gateway.execmodel.backend,
         }
-        gateway._send(Message.CHANNEL_DATA, message.channelid,
-                      dumps_internal(d))
+        gateway._send(Message.CHANNEL_DATA, message.channelid, dumps_internal(d))
         gateway._send(Message.CHANNEL_CLOSE, message.channelid)
 
     def channel_exec(message, gateway):
@@ -483,32 +496,37 @@ def _setupmessages():
         target._strconfig = loads_internal(message.data, gateway)
 
     types = [
-        status, reconfigure, gateway_terminate,
-        channel_exec, channel_data, channel_close,
-        channel_close_error, channel_last_message,
+        status,
+        reconfigure,
+        gateway_terminate,
+        channel_exec,
+        channel_data,
+        channel_close,
+        channel_close_error,
+        channel_last_message,
     ]
     for i, handler in enumerate(types):
         Message._types.append(handler)
         setattr(Message, handler.__name__.upper(), i)
 
+
 _setupmessages()
 
 
-def geterrortext(excinfo,
-                 format_exception=traceback.format_exception, sysex=sysex):
+def geterrortext(excinfo, format_exception=traceback.format_exception, sysex=sysex):
     try:
         l = format_exception(*excinfo)
         errortext = "".join(l)
     except sysex:
         raise
     except:
-        errortext = '%s: %s' % (excinfo[0].__name__,
-                                excinfo[1])
+        errortext = "{}: {}".format(excinfo[0].__name__, excinfo[1])
     return errortext
 
 
 class RemoteError(Exception):
     """ Exception containing a stringified error from the other side. """
+
     def __init__(self, formatted):
         self.formatted = formatted
         Exception.__init__(self)
@@ -517,13 +535,12 @@ class RemoteError(Exception):
         return self.formatted
 
     def __repr__(self):
-        return "%s: %s" % (self.__class__.__name__, self.formatted)
+        return "{}: {}".format(self.__class__.__name__, self.formatted)
 
     def warn(self):
         if self.formatted != INTERRUPT_TEXT:
             # XXX do this better
-            sys.stderr.write("[%s] Warning: unhandled %r\n"
-                             % (os.getpid(), self,))
+            sys.stderr.write("[%s] Warning: unhandled %r\n" % (os.getpid(), self))
 
 
 class TimeoutError(IOError):
@@ -544,7 +561,7 @@ class Channel(object):
         assert isinstance(id, int)
         self.gateway = gateway
         # XXX: defaults copied from Unserializer
-        self._strconfig = getattr(gateway, '_strconfig', (True, False))
+        self._strconfig = getattr(gateway, "_strconfig", (True, False))
         self.id = id
         self._items = self.gateway.execmodel.queue.Queue()
         self._closed = False
@@ -567,7 +584,7 @@ class Channel(object):
         _callbacks = self.gateway._channelfactory._callbacks
         with self.gateway._receivelock:
             if self._items is None:
-                raise IOError("%r has callback already registered" % (self,))
+                raise IOError("{!r} has callback already registered".format(self))
             items = self._items
             self._items = None
             while 1:
@@ -575,11 +592,7 @@ class Channel(object):
                     olditem = items.get(block=False)
                 except self.gateway.execmodel.queue.Empty:
                     if not (self._closed or self._receiveclosed.isSet()):
-                        _callbacks[self.id] = (
-                            callback,
-                            endmarker,
-                            self._strconfig,
-                        )
+                        _callbacks[self.id] = (callback, endmarker, self._strconfig)
                     break
                 else:
                     if olditem is ENDMARKER:
@@ -595,7 +608,7 @@ class Channel(object):
         return "<Channel id=%d %s>" % (self.id, flag)
 
     def __del__(self):
-        if self.gateway is None:   # can be None in tests
+        if self.gateway is None:  # can be None in tests
             return
         self._trace("channel.__del__")
         # no multithreading issues here, because we have the last ref to 'self'
@@ -614,7 +627,7 @@ class Channel(object):
             # don't need to try to send a closing or last message
             # (and often it won't work anymore to send things out)
             if Message is not None:
-                if self._items is None:    # has_callback
+                if self._items is None:  # has_callback
                     msgcode = Message.CHANNEL_LAST_MESSAGE
                 else:
                     msgcode = Message.CHANNEL_CLOSE
@@ -642,7 +655,7 @@ class Channel(object):
         """
         return self._closed
 
-    def makefile(self, mode='w', proxyclose=False):
+    def makefile(self, mode="w", proxyclose=False):
         """ return a file-like object.
             mode can be 'w' or 'r' for writeable/readable files.
             if proxyclose is true file.close() will also close the channel.
@@ -651,7 +664,7 @@ class Channel(object):
             return ChannelFileWrite(channel=self, proxyclose=proxyclose)
         elif mode == "r":
             return ChannelFileRead(channel=self, proxyclose=proxyclose)
-        raise ValueError("mode %r not availabe" % (mode,))
+        raise ValueError("mode {!r} not availabe".format(mode))
 
     def close(self, error=None):
         """ close down this channel with an optional error message.
@@ -672,14 +685,13 @@ class Channel(object):
             if not self._receiveclosed.isSet():
                 put = self.gateway._send
                 if error is not None:
-                    put(Message.CHANNEL_CLOSE_ERROR, self.id,
-                        dumps_internal(error))
+                    put(Message.CHANNEL_CLOSE_ERROR, self.id, dumps_internal(error))
                 else:
                     put(Message.CHANNEL_CLOSE, self.id)
                 self._trace("sent channel close message")
             if isinstance(error, RemoteError):
                 self._remoteerrors.append(error)
-            self._closed = True         # --> "closed"
+            self._closed = True  # --> "closed"
             self._receiveclosed.set()
             queue = self._items
             if queue is not None:
@@ -713,7 +725,7 @@ class Channel(object):
         raised if the write pipe was prematurely closed.
         """
         if self.isclosed():
-            raise IOError("cannot send to %r" % (self,))
+            raise IOError("cannot send to {!r}".format(self))
         self.gateway._send(Message.CHANNEL_DATA, self.id, dumps_internal(item))
 
     def receive(self, timeout=None):
@@ -731,7 +743,7 @@ class Channel(object):
         try:
             x = itemqueue.get(timeout=timeout)
         except self.gateway.execmodel.queue.Empty:
-            raise self.TimeoutError("no item after %r seconds" % (timeout))
+            raise self.TimeoutError("no item after %r seconds" % timeout)
         if x is ENDMARKER:
             itemqueue.put(x)  # for other receivers
             raise self._getremoteerror() or EOFError()
@@ -746,6 +758,7 @@ class Channel(object):
             return self.receive()
         except EOFError:
             raise StopIteration
+
     __next__ = next
 
     def reconfigure(self, py2str_as_py3str=True, py3str_as_py2str=False):
@@ -758,6 +771,7 @@ class Channel(object):
         data = dumps_internal(self._strconfig)
         self.gateway._send(Message.RECONFIGURE, self.id, data=data)
 
+
 ENDMARKER = object()
 INTERRUPT_TEXT = "keyboard-interrupted"
 
@@ -776,7 +790,7 @@ class ChannelFactory(object):
         """ create a new Channel with 'id' (or create new id if None). """
         with self._writelock:
             if self.finished:
-                raise IOError("connexion already closed: %s" % (self.gateway,))
+                raise IOError("connexion already closed: {}".format(self.gateway))
             if id is None:
                 id = self.count
                 self.count += 2
@@ -821,7 +835,7 @@ class ChannelFactory(object):
                 queue.put(ENDMARKER)
             self._no_longer_opened(id)
             if not sendonly:  # otherwise #--> "sendonly"
-                channel._closed = True          # --> "closed"
+                channel._closed = True  # --> "closed"
             channel._receiveclosed.set()
 
     def _local_receive(self, id, data):
@@ -832,21 +846,21 @@ class ChannelFactory(object):
         except KeyError:
             queue = channel and channel._items
             if queue is None:
-                pass    # drop data
+                pass  # drop data
             else:
                 item = loads_internal(data, channel)
                 queue.put(item)
         else:
             try:
                 data = loads_internal(data, channel, strconfig)
-                callback(data)   # even if channel may be already closed
+                callback(data)  # even if channel may be already closed
             except Exception:
                 excinfo = sys.exc_info()
-                self.gateway._trace("exception during callback: %s" %
-                                    excinfo[1])
+                self.gateway._trace("exception during callback: %s" % excinfo[1])
                 errortext = self.gateway._geterrortext(excinfo)
-                self.gateway._send(Message.CHANNEL_CLOSE_ERROR,
-                                   id, dumps_internal(errortext))
+                self.gateway._send(
+                    Message.CHANNEL_CLOSE_ERROR, id, dumps_internal(errortext)
+                )
                 self._local_close(id, errortext)
 
     def _finished_receiving(self):
@@ -871,8 +885,8 @@ class ChannelFile(object):
             self.channel.close()
 
     def __repr__(self):
-        state = self.channel.isclosed() and 'closed' or 'open'
-        return '<ChannelFile %d %s>' % (self.channel.id, state)
+        state = self.channel.isclosed() and "closed" or "open"
+        return "<ChannelFile %d %s>" % (self.channel.id, state)
 
 
 class ChannelFileWrite(ChannelFile):
@@ -907,8 +921,8 @@ class ChannelFileRead(ChannelFile):
         if self._buffer is not None:
             i = self._buffer.find("\n")
             if i != -1:
-                return self.read(i+1)
-            line = self.read(len(self._buffer)+1)
+                return self.read(i + 1)
+            line = self.read(len(self._buffer) + 1)
         else:
             line = self.read(1)
         while line and line[-1] != "\n":
@@ -928,8 +942,7 @@ class BaseGateway(object):
         self.execmodel = io.execmodel
         self._io = io
         self.id = id
-        self._strconfig = (Unserializer.py2str_as_py3str,
-                           Unserializer.py3str_as_py2str)
+        self._strconfig = (Unserializer.py2str_as_py3str, Unserializer.py3str_as_py2str)
         self._channelfactory = ChannelFactory(self, _startcount)
         self._receivelock = self.execmodel.RLock()
         # globals may be NONE at process-termination
@@ -963,16 +976,16 @@ class BaseGateway(object):
             self._error = self.exc_info()[1]
         except Exception:
             log(self._geterrortext(self.exc_info()))
-        log('finishing receiving thread')
+        log("finishing receiving thread")
         # wake up and terminate any execution waiting to receive
         self._channelfactory._finished_receiving()
-        log('terminating execution')
+        log("terminating execution")
         self._terminate_execution()
-        log('closing read')
+        log("closing read")
         self._io.close_read()
-        log('closing write')
+        log("closing write")
         self._io.close_write()
-        log('terminating our receive pseudo pool')
+        log("terminating our receive pseudo pool")
         self._receivepool.trigger_shutdown()
 
     def _terminate_execution(self):
@@ -982,10 +995,10 @@ class BaseGateway(object):
         message = Message(msgcode, channelid, data)
         try:
             message.to_io(self._io)
-            self._trace('sent', message)
+            self._trace("sent", message)
         except (IOError, ValueError):
             e = sys.exc_info()[1]
-            self._trace('failed to send', message, e)
+            self._trace("failed to send", message, e)
             # ValueError might be because the IO is already closed
             raise IOError("cannot send (already closed?)")
 
@@ -1008,18 +1021,16 @@ class BaseGateway(object):
 
 
 class SlaveGateway(BaseGateway):
-
     def _local_schedulexec(self, channel, sourcetask):
         sourcetask = loads_internal(sourcetask)
-        self._execpool.spawn(self.executetask, ((channel, sourcetask)))
+        self._execpool.spawn(self.executetask, (channel, sourcetask))
 
     def _terminate_execution(self):
         # called from receiverthread
         self._trace("shutting down execution pool")
         self._execpool.trigger_shutdown()
         if not self._execpool.waitall(5.0):
-            self._trace(
-                "execution ongoing after 5 secs,"" trying interrupt_main")
+            self._trace("execution ongoing after 5 secs," " trying interrupt_main")
             # We try hard to terminate execution based on the assumption
             # that there is only one gateway object running per-process.
             if sys.platform != "win32":
@@ -1029,13 +1040,15 @@ class SlaveGateway(BaseGateway):
                 self._trace("calling interrupt_main()")
                 interrupt_main()
             if not self._execpool.waitall(10.0):
-                self._trace("execution did not finish in another 10 secs, "
-                            "calling os._exit()")
+                self._trace(
+                    "execution did not finish in another 10 secs, " "calling os._exit()"
+                )
                 os._exit(1)
 
     def serve(self):
         def trace(msg):
             self._trace("[serve] " + msg)
+
         hasprimary = self.execmodel.backend == "thread"
         self._execpool = self.execmodel.WorkerPool(hasprimary=hasprimary)
         trace("spawning receiver thread")
@@ -1053,25 +1066,24 @@ class SlaveGateway(BaseGateway):
 
     def executetask(self, item):
         try:
-            channel, (source, call_name, kwargs) = item
+            channel, (source, file_name, call_name, kwargs) = item
             if not ISPY3 and kwargs:
                 # some python2 versions do not accept unicode keyword params
                 # note: Unserializer generally turns py2-str to py3-str objects
                 newkwargs = {}
                 for name, value in kwargs.items():
                     if isinstance(name, unicode):
-                        name = name.encode('ascii')
+                        name = name.encode("ascii")
                     newkwargs[name] = value
                 kwargs = newkwargs
-            loc = {'channel': channel, '__name__': '__channelexec__'}
-            self._trace("execution starts[%s]: %s" %
-                        (channel.id, repr(source)[:50]))
+            loc = {"channel": channel, "__name__": "__channelexec__"}
+            self._trace("execution starts[%s]: %s" % (channel.id, repr(source)[:50]))
             channel._executing = True
             try:
-                co = compile(source+'\n', '<remote exec>', 'exec')
+                co = compile(source + "\n", file_name or "<remote exec>", "exec")
                 do_exec(co, loc)  # noqa
                 if call_name:
-                    self._trace('calling %s(**%60r)' % (call_name, kwargs))
+                    self._trace("calling %s(**%60r)" % (call_name, kwargs))
                     function = loc[call_name]
                     function(channel, **kwargs)
             finally:
@@ -1084,13 +1096,14 @@ class SlaveGateway(BaseGateway):
             excinfo = self.exc_info()
             if not isinstance(excinfo[1], EOFError):
                 if not channel.gateway._channelfactory.finished:
-                    self._trace("got exception: %r" % (excinfo[1],))
+                    self._trace("got exception: {!r}".format(excinfo[1]))
                     errortext = self._geterrortext(excinfo)
                     channel.close(errortext)
                     return
             self._trace("ignoring EOFError because receiving finished")
         channel.close()
 
+
 #
 # Cross-Python pickling code, tested from test_serializer.py
 #
@@ -1109,8 +1122,11 @@ class LoadError(DataFormatError):
 
 
 if ISPY3:
+
     def bchr(n):
         return bytes([n])
+
+
 else:
     bchr = chr
 
@@ -1131,15 +1147,15 @@ class _Stop(Exception):
 class Unserializer(object):
     num2func = {}  # is filled after this class definition
     py2str_as_py3str = True  # True
-    py3str_as_py2str = False   # false means py2 will get unicode
+    py3str_as_py2str = False  # false means py2 will get unicode
 
     def __init__(self, stream, channel_or_gateway=None, strconfig=None):
-        gateway = getattr(channel_or_gateway, 'gateway', channel_or_gateway)
-        strconfig = getattr(channel_or_gateway, '_strconfig', strconfig)
+        gateway = getattr(channel_or_gateway, "gateway", channel_or_gateway)
+        strconfig = getattr(channel_or_gateway, "_strconfig", strconfig)
         if strconfig:
             self.py2str_as_py3str, self.py3str_as_py2str = strconfig
         self.stream = stream
-        self.channelfactory = getattr(gateway, '_channelfactory', gateway)
+        self.channelfactory = getattr(gateway, "_channelfactory", gateway)
 
     def load(self, versioned=False):
         if versioned:
@@ -1156,8 +1172,8 @@ class Unserializer(object):
                     loader = self.num2func[opcode]
                 except KeyError:
                     raise LoadError(
-                        "unkown opcode %r - "
-                        "wire protocol corruption?" % (opcode,))
+                        "unkown opcode %r - " "wire protocol corruption?" % (opcode,)
+                    )
                 loader(self)
         except _Stop:
             if len(self.stack) != 1:
@@ -1187,6 +1203,7 @@ class Unserializer(object):
         load_long = load_int
         load_longlong = load_longint
     else:
+
         def load_long(self):
             i = self._read_int4()
             self.stack.append(long(i))
@@ -1274,6 +1291,7 @@ class Unserializer(object):
         newchannel = self.channelfactory.new(id)
         self.stack.append(newchannel)
 
+
 # automatically build opcodes and byte-encoding
 
 
@@ -1283,9 +1301,7 @@ class opcode:
 
 def _buildopcodes():
     l = []
-    later_added = {
-        'COMPLEX': 1,
-    }
+    later_added = {"COMPLEX": 1}
     for name, func in Unserializer.__dict__.items():
         if name.startswith("load_"):
             opname = name[5:].upper()
@@ -1294,10 +1310,11 @@ def _buildopcodes():
 
     for i, (opname, func) in enumerate(l):
         assert i < 26, "xxx"
-        i = bchr(64+i)
+        i = bchr(64 + i)
         Unserializer.num2func[i] = func
         setattr(opcode, opname, i)
 
+
 _buildopcodes()
 
 
@@ -1331,9 +1348,9 @@ def loads(bytestring, py2str_as_py3str=False, py3str_as_py2str=False):
     ``execnet.DataFormatError`` will be raised.
     """
     io = BytesIO(bytestring)
-    return load(io,
-                py2str_as_py3str=py2str_as_py3str,
-                py3str_as_py2str=py3str_as_py2str)
+    return load(
+        io, py2str_as_py3str=py2str_as_py3str, py3str_as_py2str=py3str_as_py2str
+    )
 
 
 def load(io, py2str_as_py3str=False, py3str_as_py2str=False):
@@ -1382,10 +1399,10 @@ class _Serializer(object):
         try:
             dispatch = self._dispatch[tp]
         except KeyError:
-            methodname = 'save_' + tp.__name__
+            methodname = "save_" + tp.__name__
             meth = getattr(self.__class__, methodname, None)
             if meth is None:
-                raise DumpError("can't serialize %s" % (tp,))
+                raise DumpError("can't serialize {}".format(tp))
             dispatch = self._dispatch[tp] = meth
         dispatch(self, obj)
 
@@ -1403,10 +1420,13 @@ class _Serializer(object):
         self._write_byte_sequence(bytes_)
 
     if ISPY3:
+
         def save_str(self, s):
             self._write(opcode.PY3STRING)
             self._write_unicode_string(s)
+
     else:
+
         def save_str(self, s):
             self._write(opcode.PY2STRING)
             self._write_byte_sequence(s)
@@ -1448,8 +1468,7 @@ class _Serializer(object):
         self._write(opcode.COMPLEX)
         self._write(struct.pack(COMPLEX_FORMAT, cpx.real, cpx.imag))
 
-    def _write_int4(self, i, error="int must be less than %i" %
-                    (FOUR_BYTE_INT_MAX,)):
+    def _write_int4(self, i, error="int must be less than %i" % (FOUR_BYTE_INT_MAX,)):
         if i > FOUR_BYTE_INT_MAX:
             raise DumpError(error)
         self._write(struct.pack("!i", i))
@@ -1494,41 +1513,42 @@ class _Serializer(object):
 
 
 def init_popen_io(execmodel):
-    if not hasattr(os, 'dup'):  # jython
+    if not hasattr(os, "dup"):  # jython
         io = Popen2IO(sys.stdout, sys.stdin, execmodel)
         import tempfile
-        sys.stdin = tempfile.TemporaryFile('r')
-        sys.stdout = tempfile.TemporaryFile('w')
+
+        sys.stdin = tempfile.TemporaryFile("r")
+        sys.stdout = tempfile.TemporaryFile("w")
     else:
         try:
             devnull = os.devnull
         except AttributeError:
-            if os.name == 'nt':
-                devnull = 'NUL'
+            if os.name == "nt":
+                devnull = "NUL"
             else:
-                devnull = '/dev/null'
+                devnull = "/dev/null"
         # stdin
-        stdin = execmodel.fdopen(os.dup(0), 'r', 1)
+        stdin = execmodel.fdopen(os.dup(0), "r", 1)
         fd = os.open(devnull, os.O_RDONLY)
         os.dup2(fd, 0)
         os.close(fd)
 
         # stdout
-        stdout = execmodel.fdopen(os.dup(1), 'w', 1)
+        stdout = execmodel.fdopen(os.dup(1), "w", 1)
         fd = os.open(devnull, os.O_WRONLY)
         os.dup2(fd, 1)
 
         # stderr for win32
-        if os.name == 'nt':
-            sys.stderr = execmodel.fdopen(os.dup(2), 'w', 1)
+        if os.name == "nt":
+            sys.stderr = execmodel.fdopen(os.dup(2), "w", 1)
             os.dup2(fd, 2)
         os.close(fd)
         io = Popen2IO(stdout, stdin, execmodel)
-        sys.stdin = execmodel.fdopen(0, 'r', 1)
-        sys.stdout = execmodel.fdopen(1, 'w', 1)
+        sys.stdin = execmodel.fdopen(0, "r", 1)
+        sys.stdout = execmodel.fdopen(1, "w", 1)
     return io
 
 
 def serve(io, id):
-    trace("creating slavegateway on %r" % (io,))
+    trace("creating slavegateway on {!r}".format(io))
     SlaveGateway(io=io, id=id, _startcount=2).serve()
